@@ -1,5 +1,5 @@
 # **********************************************************
-# Copyright (c) 2005-2022 VMware, Inc.
+# Copyright (c) 2005-2023 VMware, Inc.
 # **********************************************************
 
 import base64
@@ -16,7 +16,7 @@ import threading
 import time
 from datetime import datetime
 from xml.parsers.expat import ExpatError, ParserCreate
-# For Visor, space is very limited. Import xml.sax pull in too many junk.
+# For Visor, space is very limited. Import xml.sax pull in too much junk.
 # Define our own xml escape instead
 # from xml.sax.saxutils import escape
 
@@ -191,7 +191,7 @@ def SetHandlers(obj, handlers):
 # @return the serialized object as bytes
 # @param encoding Deprecated this is not used during serialization since we
 #        always use utf-8 to encode a request message. We didn't remove the
-#        parameter so it is still compatible with clients that are still
+#        parameter, so it is still compatible with clients that are still
 #        using it.
 def Serialize(val,
               info=None,
@@ -305,7 +305,7 @@ class SoapSerializer:
     # @param nsMap a dict of xml ns -> prefix
     # @param encoding Deprecated this is not used during serialization since we
     #                 always use utf-8 to encode a request message. We didn't
-    #                 remove the parameter so it is still compatible with
+    #                 remove the parameter, so it is still compatible with
     #                 clients that are still using it.
     def __init__(self, writer, version, nsMap, encoding=None):
         """ Constructor """
@@ -566,7 +566,7 @@ class SoapSerializer:
             self.writer.write(u'<{0}{1}>{2}</{0}>'.format(
                 info.name, attr, result))
 
-    # Serialize a a data object (internal)
+    # Serialize a data object (internal)
     #
     # @param val the value to serialize
     # @param info the field
@@ -605,7 +605,7 @@ class SoapSerializer:
 # This function will deserialize one top-level XML node.
 # @param data the data to deserialize (a file object or string)
 # @param resultType expected result type
-# @param stub stub for moRef deserialization
+# @param stub the stub for moRef deserialization
 # @return the deserialized object
 def Deserialize(data, resultType=object, stub=None):
     parser = ParserCreate(namespace_separator=NS_SEP)
@@ -672,8 +672,8 @@ class ExpatDeserializerNSHandlers:
 class SoapDeserializer(ExpatDeserializerNSHandlers):
     # Constructor
     #
-    # @param self self
-    # @param stub Stub adapter to use for deserializing moRefs
+    # @param self The object pointer
+    # @param stub The stub adapter to use for deserializing moRefs
     def __init__(self, stub=None, version=None):
         ExpatDeserializerNSHandlers.__init__(self)
         self.stub = stub
@@ -687,7 +687,7 @@ class SoapDeserializer(ExpatDeserializerNSHandlers):
 
     # Deserialize a SOAP object
     #
-    # @param self self
+    # @param self The object pointer
     # @param parser an expat parser
     # @param resultType the static type of the result
     # @param isFault true if the response is a fault response
@@ -912,8 +912,8 @@ class SoapDeserializer(ExpatDeserializerNSHandlers):
 class SoapResponseDeserializer(ExpatDeserializerNSHandlers):
     # Constructor
     #
-    # @param self self
-    # @param stub Stub adapter to use for deserializing moRefs
+    # @param self The object pointer
+    # @param stub The stub adapter to use for deserializing moRefs
     def __init__(self, stub):
         ExpatDeserializerNSHandlers.__init__(self)
         self.stub = stub
@@ -922,7 +922,7 @@ class SoapResponseDeserializer(ExpatDeserializerNSHandlers):
 
     # Deserialize a SOAP response
     #
-    # @param self self
+    # @param self The object pointer
     # @param response the response (a file object or a string)
     # @param resultType expected result type
     # @param nsMap a dict of prefix -> [xml ns stack]
@@ -987,13 +987,14 @@ class SoapResponseDeserializer(ExpatDeserializerNSHandlers):
 # -- InvokeMethod(ManagedObject mo, Object methodInfo, Object[] args)
 class StubAdapterBase(StubAdapterAccessorMixin):
     def __init__(self, version):
+        StubAdapterAccessorMixin.__init__(self)
         self.ComputeVersionInfo(version)
 
     # Compute the version information for the specified namespace
     #
     # @param ns the namespace
     def ComputeVersionInfo(self, version):
-        # Make sure we do NOT fallback to an older version
+        # Make sure we do NOT fall back to an older version
         if hasattr(self, 'version') and IsChildVersion(self.version, version):
             # print("WARNING: stub degrading: " +
             #       self.version + " -> " + version)
@@ -1094,10 +1095,10 @@ class UnixSocketConnection(six.moves.http_client.HTTPConnection):
 
 
 def _VerifyThumbprint(thumbprint, connection):
-    '''If there is a thumbprint, connect to the server and verify that the
+    """If there is a thumbprint, connect to the server and verify that the
     SSL certificate matches the given thumbprint.  An exception is thrown
     if there is a mismatch.
-    '''
+    """
     if thumbprint and isinstance(connection,
                                  six.moves.http_client.HTTPSConnection):
         if not connection.sock:
@@ -1166,8 +1167,10 @@ class _HTTPSConnection(six.moves.http_client.HTTPSConnection):
 # issue a CONNECT command to start an SSL tunnel.
 class SSLTunnelConnection(object):
     # @param proxyPath The path to pass to the CONNECT command.
-    def __init__(self, proxyPath):
+    # @param customHeaders Dictionary with custom HTTP headers.
+    def __init__(self, proxyPath, customHeaders=None):
         self.proxyPath = proxyPath
+        self.customHeaders = customHeaders if customHeaders else {}
 
     # Connects to a proxy server and initiates a tunnel to the destination
     # specified by proxyPath. If successful, a new HTTPSConnection is returned.
@@ -1227,7 +1230,7 @@ class SSLTunnelConnection(object):
             proxyHost = splitport(self.proxyPath)[0]
             if (_CheckIPv4(proxyHost) or _CheckIPv6(proxyHost)
                     or _CheckHostname(proxyHost)):
-                retval.set_tunnel(self.proxyPath)
+                retval.set_tunnel(self.proxyPath, headers=self.customHeaders)
             # Call wrap_socket if ProxyPath is VC inbuilt proxyPath
             # ex: /sdkTunnel
             else:
@@ -1247,8 +1250,10 @@ class SSLTunnelConnection(object):
 # proxy.
 class HTTPProxyConnection(object):
     # @param proxyPath The path to pass to the CONNECT command.
-    def __init__(self, proxyPath):
+    # @param customHeaders Dictionary with custom HTTP headers.
+    def __init__(self, proxyPath, customHeaders=None):
         self.proxyPath = proxyPath
+        self.customHeaders = customHeaders if customHeaders else {}
 
     # Connects to an HTTP proxy server and initiates a tunnel to the destination
     # specified by proxyPath. If successful, a new HTTPSConnection is returned.
@@ -1259,7 +1264,7 @@ class HTTPProxyConnection(object):
     def __call__(self, path, **kwargs):
         httpsConnArgs = {k: kwargs[k] for k in kwargs if k not in SOAP_ADAPTER_ARGS}
         conn = six.moves.http_client.HTTPSConnection(path, **httpsConnArgs)
-        conn.set_tunnel(self.proxyPath)
+        conn.set_tunnel(self.proxyPath, headers=self.customHeaders)
         return conn
 
 
@@ -1323,7 +1328,7 @@ class GzipReader:
             leftoverChunks = []
         else:
             leftoverBytes = bufSize - bytes
-            # Adjust last chunk to hold only the left over bytes
+            # Adjust last chunk to hold only the leftover bytes
             lastChunk = chunks.pop()
             chunks.append(lastChunk[:-leftoverBytes])
             leftoverChunks = [lastChunk[-leftoverBytes:]]
@@ -1341,7 +1346,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
     #
     # The endpoint can be specified individually as either a host/port
     # combination, or with a URL (using a url= keyword).
-    # @param self self
+    # @param self The object pointer
     # @param host host
     # @param port port (pass negative port number for no SSL)
     # @param **** Deprecated. Please use version instead **** ns API namespace
@@ -1440,10 +1445,10 @@ class SoapStubAdapter(SoapStubAdapterBase):
 
         self.is_tunnel = False
         if sslProxyPath:
-            self.scheme = SSLTunnelConnection(sslProxyPath)
+            self.scheme = SSLTunnelConnection(sslProxyPath, customHeaders)
             self.is_tunnel = True
         elif httpProxyHost:
-            self.scheme = HTTPProxyConnection(self.host)
+            self.scheme = HTTPProxyConnection(self.host, customHeaders)
             self.is_tunnel = True
 
             # Is httpProxyHost IPv6
@@ -1479,7 +1484,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
 
     # Context modifier used to modify the SOAP request.
     # @param func The func that takes in the serialized message and
-    #   modifies the the request. The func is appended to the
+    #   modifies the request. The func is appended to the
     #   requestModifierList and then popped after the request is modified.
     @contextlib.contextmanager
     def requestModifier(self, func):
@@ -1491,7 +1496,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
 
     # Invoke a managed method
     #
-    # @param self self
+    # @param self The object pointer
     # @param mo the 'this'
     # @param info method info
     # @param args arguments
@@ -1528,7 +1533,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
             conn.request('POST', self.path, req, headers)
             resp = conn.getresponse()
         except (socket.error, six.moves.http_client.HTTPException):
-            # The server is probably sick, drop all of the cached connections.
+            # The server is probably sick, drop all the cached connections.
             self.DropConnections()
             raise
         cookie = resp.getheader('Set-Cookie')
@@ -1553,7 +1558,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
             # TODO Add specific exception(s)
             except:  # noqa: E722
                 conn.close()
-                # The server might be sick, drop all of the cached connections.
+                # The server might be sick, drop all the cached connections.
                 self.DropConnections()
                 raise
             else:
@@ -1572,7 +1577,10 @@ class SoapStubAdapter(SoapStubAdapterBase):
                                  "  Attributes: {}\n  Hierarchy: {}")
                     msg = formatMsg.format(_dict, str(inheritanceTree))
                     raise Exception(msg)
-                raise obj  # pylint: disable-msg=E0702
+                try:
+                    raise obj  # pylint: disable-msg=E0702
+                finally:
+                    del obj
         else:
             conn.close()
             raise six.moves.http_client.HTTPException(
@@ -1595,7 +1603,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
             for conn, _ in idleConnections:
                 conn.close()
 
-    # Get a HTTP connection from the pool
+    # Get an HTTP connection from the pool
     def GetConnection(self):
         self.lock.acquire()
         self._CloseIdleConnections()
@@ -1619,7 +1627,7 @@ class SoapStubAdapter(SoapStubAdapterBase):
         for conn, _ in oldConnections:
             conn.close()
 
-    # Return a HTTP connection to the pool
+    # Return an HTTP connection to the pool
     def ReturnConnection(self, conn):
         self.lock.acquire()
         self._CloseIdleConnections()
@@ -1656,11 +1664,11 @@ def ParseHttpResponse(httpResponse):
 
 
 class SessionOrientedStub(StubAdapterBase):
-    '''A session-oriented stub adapter that will relogin to the destination if
+    """A session-oriented stub adapter that will relogin to the destination if
     a session-oriented exception is thrown.
 
 
-    Here's an example.  First, we setup the communication substrate:
+    Here's an example.  First, we set up the communication substrate:
 
     >>> soapStub = SoapStubAdapter(host="192.168.1.2", ns="vim25/5.5")
 
@@ -1678,7 +1686,7 @@ class SessionOrientedStub(StubAdapterBase):
     >>> si.content.sessionManager.sessionList
     >>> si.content.sessionManager.Logout()
     >>> si.content.sessionManager.sessionList
-    '''
+    """
 
     STATE_UNAUTHENTICATED = 0
     STATE_AUTHENTICATED = 1
@@ -1686,7 +1694,7 @@ class SessionOrientedStub(StubAdapterBase):
     SESSION_EXCEPTIONS = tuple()
 
     def __init__(self, soapStub, loginMethod, retryDelay=0.1, retryCount=4):
-        '''Construct a SessionOrientedStub.
+        """Construct a SessionOrientedStub.
 
         The stub starts off in the "unauthenticated" state, so it will
         call the loginMethod on the first invocation of a method.  If a
@@ -1705,7 +1713,7 @@ class SessionOrientedStub(StubAdapterBase):
             communication error.
         @param retryCount The number of times to retry connecting to the
             server.
-        '''
+        """
         assert callable(loginMethod)
         assert retryCount >= 0
         StubAdapterBase.__init__(self, version=soapStub.version)
@@ -1759,7 +1767,7 @@ class SessionOrientedStub(StubAdapterBase):
 
     # Retrieve a managed property
     #
-    # @param self self
+    # @param self The object pointer
     # @param mo managed object
     # @param info property info
     def InvokeAccessor(self, mo, info):
